<?php

namespace HCM\FPA\ObrigacaoLegal\Dirf;

use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\ObrigacaoLegal\Dirf\QueryDirf;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroDIRF;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroIDREC;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroBPFDEC;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroMensal;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroRIO;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroINFPA;
use HCM\Util\DateUtil;
use HCM\Util\MsgCalculo;
use HCM\Util\Repositories;

class Dirf2021 {

	public function __construct($entityManager, $hcmParameters, $objMagnetico, $nrRegimeApur, $dsOrdem, $dsInicio, $dsFim, $idVincRetencao, $idVincRendimento, $vrMinRendimento, $idApuraBeneficio){
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador = $hcmParameters->getCdOperador();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $this->_nrTipoModalidadeCalculo = $hcmParameters->getTipoModalidadeCalculo();
        
        $this->_calculoFolha = new FpacalculofolhaEntity($entityManager, $hcmParameters);
        $this->_itemCalculo = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
        $this->_evento = new FpaeventomEntity($entityManager, $hcmParameters);
        
        $this->_objMagnetico = $objMagnetico;
        $this->_nrRegimeApur = $nrRegimeApur;
        $this->_dsOrdem = $dsOrdem;
        $this->_dsInicio = $dsInicio;
        $this->_dsFim = $dsFim;
        
        $this->_idVincRetencao = $idVincRetencao;
        $this->_idVincRendimento= $idVincRendimento;
        $this->_vrMinRendimento= $vrMinRendimento;
        $this->_idApuraBeneficio = $idApuraBeneficio;

        $this->_registroIDREC = new RegistroIDREC();
        $this->_registroBPFDEC = new RegistroBPFDEC();
        $this->_registroMensal = new RegistroMensal();
        $this->_registroRIO = new RegistroRIO();
        $this->_registroINFPA = new RegistroINFPA();
        
        $this->queryDirf = new QueryDirf($this->entityManager,$this->hcmParameters);
    }

    public function geraDIRF($nrpessoa){
        $this->_pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaAtual(
            $this->_nrOrg, $nrpessoa, $this->_mesCompetencia
        );
        
        $this->_idAlimentado = 'N';

        $this->_vrTotalRTRT = 0;
        $this->_vrTotalRTRT13 = 0;
        $this->_vrTotalRTIRF = 0;
        $this->_dsComplemento = null;
        
        $this->_dadoVinculoRec = array();
        $this->_dadoPse = array();

        $this->_calculaDIRF();

        if (count($this->_dadoVinculoRec) > 0) {
            foreach ($this->_dadoVinculoRec as $receita => $dadoVinculo) {
                if (!is_null($receita)) {
                    $this->_dadoVinculo = $dadoVinculo;
                    
                    $this->_preparaRegistroIDREC($receita);
                    
                    $this->_preparaRegistroBPFDEC();
                    
                    if ($this->_idAlimentado == 'S') {
                        $this->_preparaRegistroINFPA();
                    }
                    
                    $this->_preparaRegistroRTPA();
                    $this->_preparaRegistroRIIRP();
                    $this->_preparaRegistroRIAP();
                    $this->_preparaRegistroRIDAC();
                    $this->_preparaRegistroRTDP();
                    $this->_preparaRegistroRIO();
                    $this->_preparaRegistroRTIRF();
                    $this->_preparaRegistroRTPO();
                    $this->_preparaRegistroRTRT();
                   
                    $this->_objMagnetico->fimAddRegVinculo();
                }
            }
            
            return $this->_pessoa->historico->getNrcpfpessoa();
        }
        
        return null;
    }

    private function _calculaDIRF()
    {

        $this->_calculaRTRT();
        
        //Se estiver marcado no formulário para somente vinculos com rendimento superior a X
        if ($this->_idVincRendimento == 'S') {
            if ($this->_vrTotalRTRT < $this->_vrMinRendimento) {
                //Retorna o array vazio para não gerar o vínculo
                $this->_dadoVinculoRec = array();
                return;
            }
        }
        
        $this->_calculaRTIRF();
        
        //Se estiver marcado no formulário para somente vinculos com retenção
        if ($this->_idVincRetencao == 'S') {
            if ($this->_vrTotalRTIRF = 0) {
                //Retorna o array vazio para não gerar o vínculo
                $this->_dadoVinculoRec = array();
                return;
            }
        }
        
        $this->_calculaRTPO();
        $this->_calculaRTDP();
        $this->_calculaRTPA();
        $this->_calculaRIDAC();
        $this->_calculaRIIRP();
        $this->_calculaRIAP();
        $this->_calculaRIO();
    }

    private function _consolidaDadosVinculo($receita, $referencia, $mes, $valor)
    {
        $this->_dadoVinculoRec[$receita][$referencia][$mes] += $valor;
        if ($mes == 13) {
            $this->_dadoVinculoRec[$receita][$referencia]['VRTOTAL13'] += $valor;
        } else {
            $this->_dadoVinculoRec[$receita][$referencia]['VRTOTAL'] += $valor;
        }
    }
    
    private function _calculaValorMensal($nritem, $evento)
    {
        $ano = substr($evento['MES'],0,4);
        $mes = (int) substr($evento['MES'],4,2);

        if ($this->_mesCompetencia->format('Y') == $ano && (float)$evento['VREVENTOFOLHA'] != 0 && strpos($evento['NMITEM'],'-INF') === false) {
            
            $this->_consolidaDadosVinculo($evento['CDRECEITA'], $nritem, $mes, $evento['VREVENTOFOLHA']);

            if ($nritem == RegistroDIRF::ITEM_RTRT) {
                if ($mes == 13) {
                    $this->_vrTotalRTRT13 += $evento['VREVENTOFOLHA'];
                } else {
                    $this->_vrTotalRTRT += $evento['VREVENTOFOLHA'];
                }
                
            }
            
            if ($nritem == RegistroDIRF::ITEM_RTIRF) {
                $this->_vrTotalRTIRF += $evento['VREVENTOFOLHA'];
            }
        }
    }
    
    private function _retornaItensRegistrosMensais($nmregistro)
    {
        $bases = $this->_itemCalculo->retornaItensPorVinculoDirf(
            $this->_nrOrg, $this->_pessoa->getNrpessoa(), $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, $this->_nrRegimeApur, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim, $nmregistro
        );
        return $bases;
    }

    private function _calculaRTRT()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RTRT);

        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RTRT, $evento);
        }
    }

    private function _calculaRTPO()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RTPO);
        
        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RTPO, $evento);
        }
    }

    private function _calculaRTDP()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RTDP);

        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RTDP, $evento);
        }
    }

    private function _calculaRTIRF()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RTIRF);

        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RTIRF, $evento);
        }
    }

    private function _calculaRTPA()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RTPA);

        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RTPA, $evento);
        }
    }

    private function _calculaRIDAC()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RIDAC);

        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RIDAC, $evento);
        }
    }

    private function _calculaRIIRP()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RIIRP);

        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RIIRP, $evento);
        }
    }

    private function _calculaRIAP()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RIAP);

        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RIAP, $evento);
        }
    }

    private function _calculaRIO()
    {
        $bases = $this->_retornaItensRegistrosMensais(RegistroDIRF::REGISTRO_RIO);
        
        foreach ($bases as $evento) {
            $this->_calculaValorMensal(RegistroDIRF::ITEM_RIO, $evento);
        }
    }

    private function _preparaRegistroIDREC($cdReceita){

        $this->_registroIDREC->setCdreceita($cdReceita);

        $this->_registroIDREC->geraRegistroIDREC();

        $this->_idrec = $this->_registroIDREC->getLinhaRegistro();

        $this->_registroIDREC->setLinhaRegistro();

    }

    private function _preparaRegistroBPFDEC(){
        
        try{
            $params = array(
                "nrorg"         => $this->_pessoa->getNrorg(),
                "cpftitular"    => $this->_pessoa->historico->getNrcpfpessoa(),
                "dtcompetencia" => $this->_mesCompetencia->format('d/m/Y'),
            );
			$sqlResult = $this->queryDirf->get_infpa_aux($params);
		
		    if(count($sqlResult[0]) > 0) { //Testa se existe dependentes
		        $this->_idAlimentado = 'S';
		    }
		} catch(\Exception $e){
			throw new \Exception('Erro na query auxiliar, campo INFPA, iten 3.17.');
		}

        $this->_registroBPFDEC->setCpf($this->_pessoa->historico->getNrcpfpessoa());
        $this->_registroBPFDEC->setNome($this->_pessoa->historico->getNmpessoa());
        $this->_registroBPFDEC->setDtmolestia(null);
        $this->_registroBPFDEC->setIdalimentado($this->_idAlimentado);
        $this->_registroBPFDEC->setIdprevidencia('N');

        $this->_registroBPFDEC->geraRegistroBPFDEC();

        $this->_bpfdec = $this->_registroBPFDEC->getLinhaRegistro();

        $this->_registroBPFDEC->setLinhaRegistro();

    }

    private function _preparaRegistroRTRT(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTRT])) {
            $this->_registroMensal->geraRegistroRTRT($this->_dadoVinculo[RegistroDIRF::ITEM_RTRT]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RTRT, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RTRT]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RTRT]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRTPO(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTPO])) {
            $this->_registroMensal->geraRegistroRTPO($this->_dadoVinculo[RegistroDIRF::ITEM_RTPO]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RTPO, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RTPO]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RTPO]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRTDP(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTDP])) {
            $this->_registroMensal->geraRegistroRTDP($this->_dadoVinculo[RegistroDIRF::ITEM_RTDP]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RTDP, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RTDP]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RTDP]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRTIRF(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTIRF])) {
            $this->_registroMensal->geraRegistroRTIRF($this->_dadoVinculo[RegistroDIRF::ITEM_RTIRF]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RTIRF, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RTIRF]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RTIRF]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRTPA(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTPA])) {
            $this->_registroMensal->geraRegistroRTPA($this->_dadoVinculo[RegistroDIRF::ITEM_RTPA]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RTPA, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RTPA]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RTPA]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRIDAC(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RIDAC])) {
            $this->_registroMensal->geraRegistroRIDAC($this->_dadoVinculo[RegistroDIRF::ITEM_RIDAC]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RIDAC, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RIDAC]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RIDAC]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRIIRP(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RIIRP])) {
            $this->_registroMensal->geraRegistroRIIRP($this->_dadoVinculo[RegistroDIRF::ITEM_RIIRP]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RIIRP, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RIIRP]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RIIRP]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRIAP(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RIAP])) {
            $this->_registroMensal->geraRegistroRIAP($this->_dadoVinculo[RegistroDIRF::ITEM_RIAP]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RIAP, $this->_registroMensal->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RIAP]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RIAP]['VRTOTAL13']
            );

            $this->_registroMensal->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRIO(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RIO])) {
            $this->_registroRIO->setDescricao('RENDIMENTOS ISENTOS - OUTROS');
                
            $this->_registroRIO->geraRegistroRIO($this->_dadoVinculo[RegistroDIRF::ITEM_RIO]);
            $this->_addRegVinculo(
                RegistroDIRF::REGISTRO_RIO, $this->_registroRIO->getLinhaRegistro(), $this->_dadoVinculo[RegistroDIRF::ITEM_RIO]['VRTOTAL'], $this->_dadoVinculo[RegistroDIRF::ITEM_RIO]['VRTOTAL13']
            );

            $this->_registroRIO->setLinhaRegistro();
        }

    }

    private function _preparaRegistroINFPA(){
        
        try{
            $params = array(
                "nrorg"         => $this->_pessoa->getNrorg(),
                "cpftitular"    => $this->_pessoa->historico->getNrcpfpessoa(),
                "dtcompetencia" => $this->_mesCompetencia->format('d/m/Y'),
            );
			$sqlResult = $this->queryDirf->get_infpa($params);
		} catch(\Exception $e){
			throw new \Exception('Erro na query auxiliar, campo INFPA, iten 3.17.');
		}
		
		foreach($sqlResult as $reg){
			if ($reg) {
			    $this->_registroINFPA->setCpf(str_replace('|','',$reg['CDCPFDEP']));
                $this->_registroINFPA->setDtnascimento(str_replace('|','',$reg['DTNASCDEP']));
                $this->_registroINFPA->setNome(str_replace('|','',$reg['NOMEDEP']));
                $this->_registroINFPA->setIdrelacdep(str_replace('|','',$reg['RELACDEP']));
        
                $this->_registroINFPA->geraRegistroINFPA();
        
                $this->_infpa = $this->_registroINFPA->getLinhaRegistro();
                $this->_addRegVinculo(RegistroDIRF::REGISTRO_INFPA, $this->_infpa);
        
                $this->_registroINFPA->setLinhaRegistro();
			}
		}

    }

    private function _addRegVinculo($nmRegistro, $linhaRegistro, $vrTotal=0, $vrTotal13=0){
        $this->_objMagnetico->addRegVinculo(
            $this->_pessoa->historico->getNrcpfpessoa(), $this->_pessoa->historico->getNmpessoa(), null, $nmRegistro, $this->_registroIDREC->getCdreceita(), $vrTotal, $vrTotal13, 
            $this->_idrec, $this->_bpfdec, $linhaRegistro, $this->_dsComplemento, null
        );
    }

}